/**
 *******************************************************************************
 * @file    tmpm384_adc.c
 * @brief   This file provides API functions for ADC driver.
 * @version V2.0.2.2
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm384_adc.h"
#include "tx03_common.h"


#if defined(__TMPM384_ADC_H)
/** @addtogroup TX03_Periph_Driver
  * @{
  */

/** @defgroup ADC
  * @brief ADC driver modules
  * @{
  */

/** @defgroup ADC_Private_Defines
  * @{
  */

#define ADC_MASK_KEEP1BIT       (uint32_t)0x01U
#define ADC_MASK_KEEP2BITS      (uint32_t)0x03U
#define ADC_MASK_KEEP3BITS      (uint32_t)0x07U
#define ADC_MASK_KEEP4BITS      (uint32_t)0x0FU
#define ADC_MASK_KEEP5BITS      (uint32_t)0x1FU
#define ADC_MASK_KEEP6BITS      (uint32_t)0x3FU
#define ADC_MASK_KEEP7BITS      (uint32_t)0x7FU
#define ADC_MASK_KEEP8BITS      (uint32_t)0xFFU

/* NOTE: MOD3 bit7 to bit0 must be wrote: 0b01 100 000 */
#define ADC_MOD3_FIX            (uint32_t)0x60U

/** @} */
/* End of group ADC_Private_Defines */

/** @defgroup ADC_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group ADC_Private_FunctionPrototypes */

/** @defgroup ADC_Private_Functions
  * @{
  */


static const volatile uint32_t *const ADC_REGx_Address[12U] = {
    &TSB_AD->REG0, &TSB_AD->REG1, &TSB_AD->REG2, &TSB_AD->REG3,
    &TSB_AD->REG4, &TSB_AD->REG5, &TSB_AD->REG6, &TSB_AD->REG7,
    &TSB_AD->REG8, &TSB_AD->REG9, &TSB_AD->REG10, &TSB_AD->REG11
};

static volatile uint32_t *const ADC_PSELx_Address[4U] = {
    &TSB_AD->PSEL0, &TSB_AD->PSEL1, &TSB_AD->PSEL2, &TSB_AD->PSEL3
};

static volatile uint32_t *const ADC_PSETx_Address[6U] = {
    &TSB_AD->PSET0, &TSB_AD->PSET1, &TSB_AD->PSET2,
    &TSB_AD->PSET3, &TSB_AD->PSET4, &TSB_AD->PSET5
};

static volatile uint32_t *const ADC_TSETx_Address[3U] = {
    &TSB_AD->TSET03, &TSB_AD->TSET47, &TSB_AD->TSET811
};

static volatile uint32_t *const ADC_SSETx_Address[3U] = {
    &TSB_AD->SSET03, &TSB_AD->SSET47, &TSB_AD->SSET811
};

static volatile uint32_t *const ADC_ASETx_Address[3U] = {
    &TSB_AD->ASET03, &TSB_AD->ASET47, &TSB_AD->ASET811
};

static uint32_t ADC_ByteClear_Mask[4U] = {
    0xFFFFFF00U, 0xFFFF00FFU, 0xFF00FFFFU, 0x00FFFFFFU
};

/** @} */
/* End of group ADC_Private_Functions */


/** @defgroup ADC_Exported_Functions
  * @{
  */

/**
  * @brief  Set ADC sample hold time and prescaler output(SCLK)
  * @param  Sample_HoldTime: Select ADC sample hold time.
  *   This parameter can be one of the following values:
  *     ADC_HOLD_FIX
  * @param  Prescaler_Output: Select ADC prescaler output.
  *   This parameter can be one of the following values:
  *     ADC_FC_DIVIDE_LEVEL_NONE
  * @retval None.
  */
void ADC_SetClk(uint32_t Sample_HoldTime, uint32_t Prescaler_Output)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_SAMPLEHOLD(Sample_HoldTime));
    assert_param(IS_ADC_PRESCALER(Prescaler_Output));

    tmp = ADC_HOLD_FIX | ADC_FC_DIVIDE_LEVEL_NONE;
    TSB_AD->CLK = tmp;
}


/**
  * @brief  Set ADC resolution. This function should be called from beginning. 
  *         When Power up reset, the ADC module is set in 12bits mode. 
  * @param  ADBits: set AD resolution as 12bits or 10bits.
  *   This parameter can be one of the following values:
  *     ADC_10BITS, ADC_12BITS.  
  * @retval None
  */
void ADC_SetResolution(ADC_Resolution ADBits)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_RESOLUTION(ADBits));

    tmp = TSB_AD->MOD3 & (ADC_MASK_KEEP1BIT << 8U);     /* only keep bit8: RCUT, */
    tmp |= (uint32_t) ADBits << 10U;    /* then OR the data wanted */
    tmp |= ADC_MOD3_FIX;        /* NOTE: bit7 to bit0 must be wrote: 0b01 100 000 */
    TSB_AD->MOD3 = tmp;
}

/**
  * @brief  Read AD resolution setting. 
  * @param  None
  * @retval Current AD resolution set, which can be:
  *     ADC_10BITS, ADC_12BITS, ADC_RESERVEDBITS.  
  */
ADC_Resolution ADC_GetResolution(void)
{
    uint32_t tmp = 0U;
    ADC_Resolution ret = ADC_RESERVEDBITS;

    /* read and mask to keep bit11 and bit10 then shift to lowest position */
    tmp = TSB_AD->MOD3 & (ADC_MASK_KEEP2BITS << 10U);
    tmp >>= 10U;

    if (tmp == ADC_10BITS) {
        ret = ADC_10BITS;
    } else if (tmp == ADC_12BITS) {
        ret = ADC_12BITS;
    } else {
        ret = ADC_RESERVEDBITS;
    }

    return ret;
}


/**
  * @brief  Enable ADC module before start ADC conversion.
  * @param  None
  * @retval None
  */
void ADC_Enable(void)
{
    uint32_t tmp = 0U;

    /* bit0 of MOD0 must be wrote '0'  */
    tmp |= (uint32_t) (0x01U << 1U);    /* then set bit1(.DACON) of ADMOD0 */
    TSB_AD->MOD0 = tmp;

    tmp = TSB_AD->MOD1 & (ADC_MASK_KEEP1BIT << 0U);     /* only keep bit0,  */
    tmp |= (uint32_t) (0x01U << 7U);    /* then set bit7(.ADEN)  of ADMOD1 */
    TSB_AD->MOD1 = tmp;
}



/**
  * @brief  Disable ADC module.
  * @param  None
  * @retval None
  */
void ADC_Disable(void)
{
    uint32_t tmp = 0U;

    /* write '0' to bit1 and bit0 of MOD0 */
    TSB_AD->MOD0 = tmp;

    tmp = TSB_AD->MOD1 & (ADC_MASK_KEEP1BIT << 0U);     /* only keep bit0 of MOD1  */
    TSB_AD->MOD1 = tmp;
}


/**
  * @brief  Start ADC conversion when select Software or Constant trigger.
  * @param  Trg: to input trigger type.  
  *   This parameter can be one of the following values: 
  *     ADC_TRG_SW, ADC_TRG_CONSTANT
  * @retval None
  */
void ADC_Start(ADC_TrgType Trg)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_START(Trg));

    if (Trg == ADC_TRG_SW) {
        tmp = TSB_AD->MOD0 & (ADC_MASK_KEEP1BIT << 1U); /* only keep bit1, */
        tmp |= (uint32_t) (0x01U << 0U);        /* then set ADMOD0.ADSS (_MOD0.bit0) */
        TSB_AD->MOD0 = tmp;
    } else {                    /* ADC_TRG_CONSTANT */
        tmp = TSB_AD->MOD1 & (ADC_MASK_KEEP1BIT << 7U); /* only keep bit7, */
        tmp |= (uint32_t) (0x01U << 0U);        /* then set ADMOD1.ADAS (_MOD1.bit0) */
        TSB_AD->MOD1 = tmp;
    }
}

/**
  * @brief  Stop ADC conversion when use Constant trigger.
  * @param  None
  * @retval None
  */
void ADC_StopConstantTrg(void)
{
    uint32_t tmp = 0U;

    /* Only ADC_TRG_CONSTANT can be stopped here */
    tmp = TSB_AD->MOD1 & (ADC_MASK_KEEP1BIT << 7U);     /* only keep bit7 */
    TSB_AD->MOD1 = tmp;
}


/**
  * @brief  Get the state of ADC conversion by input trigger information.
  * @param  Trg: to input trigger type. 
  *   This parameter can be one of the following values: 
  *     ADC_TRG_SW, ADC_TRG_CONSTANT, ADC_TRG_TIMER, ADC_TRG_PMD 
  * @retval ADC conversion state.
  *   The value returned can be one of the followings:
  *     DONE, BUSY.
  */
WorkState ADC_GetConvertState(ADC_TrgType Trg)
{
    WorkState ret = DONE;
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_TRGTYPE(Trg));

    tmp = TSB_AD->MOD2;
    if (Trg == ADC_TRG_SW) {
        tmp &= (ADC_MASK_KEEP1BIT << 1U);       /* only keep bit1 */
    } else {
        tmp &= (ADC_MASK_KEEP1BIT << 0U);       /* only keep bit0 */
    }

    if (tmp) {
        ret = BUSY;
    } else {
        /* do nothing */
    }

    return ret;
}

/**
  * @brief  Set ADC module to run in Low Power mode or Normal mode.
  *         When Power up reset, the ADC module is set in Low Power Mode. 
  * @param  NewState: to enable low power mode, or disable it to work in normal mode
  *   This parameter can be one of the following values:
  *     ENABLE , DISABLE.
  * @retval None 
  */
void ADC_SetLowPowerMode(FunctionalState NewState)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    tmp = TSB_AD->MOD3 & (ADC_MASK_KEEP2BITS << 10U);   /* only keep bit11 and bit10 */
    tmp |= ADC_MOD3_FIX;        /* NOTE: bit7 to bit0 must be wrote: 0b01 100 000 */

    if (NewState == ENABLE) {   /* if enable, Set bit8  */
        tmp |= (uint32_t) (0x01U << 8U);
    } else {
        /* do nothing */
    }

    TSB_AD->MOD3 = tmp;
}


/**
  * @brief  Set ADC monitor function and Enable it. It finally can generate interrupt
  *         INTADCPA for configration in ADCMPCR0, and INTADCPB for configration in ADCMPCR1.
  * @param  Monitor: The structure containing ADC Monitor configuration 
  *         Refer the members of ADC_MonitorTypeDef in .h file for more detail usage.  
  * @retval None 
  */
void ADC_SetMonitor(ADC_MonitorTypeDef * Monitor)
{
    uint32_t tmp = 0U;
    uint32_t tmpComp = 0U;
    ADC_Resolution res = ADC_10BITS;

    /* Check the parameters */
    assert_param(IS_POINTER_NOT_NULL(Monitor));
    assert_param(IS_ADC_CMPCRx(Monitor->CMPCRx));
    assert_param(IS_ADC_REGx(Monitor->ResultREGx));
    assert_param(IS_ADC_CMPTIMES(Monitor->CmpTimes));
    assert_param(IS_ADC_CMPCONDITION(Monitor->Condition));

    res = ADC_GetResolution();
    if (res == ADC_10BITS) {
        assert_param(IS_ADC_CMPVALUE_10BIT(Monitor->CmpValue));
        tmpComp = (Monitor->CmpValue & 0x03FFU) << 6U;  /* 10bit mode: use bit15 to bit6 */
    } else {
        assert_param(IS_ADC_CMPVALUE_12BIT(Monitor->CmpValue));
        tmpComp = (Monitor->CmpValue & 0x0FFFU) << 4U;  /* 12bit mode: use bit15 to bit4 */
    }

    tmp |= (uint32_t) (Monitor->ResultREGx);
    tmp |= (uint32_t) (Monitor->Condition) << 4U;
    tmp |= (uint32_t) (Monitor->CmpTimes - 1U) << 8U;

    tmp |= (uint32_t) (0x01U << 7U);    /* set bit 7: CMPxEN to enable Monitor function */

    if (Monitor->CMPCRx == ADC_CMPCR_0) {
        TSB_AD->CMPCR0 = tmp;
        TSB_AD->CMP0 = tmpComp;
    } else {
        TSB_AD->CMPCR1 = tmp;
        TSB_AD->CMP1 = tmpComp;
    }
}


/**
  * @brief  To disable ADC monitor function whether it has been set or not. 
  * @param  CMPCRx: Select which compare control register will be used.
  *   This parameter can be one of the following values:
  *     ADC_CMPCR_0, ADC_CMPCR_1
  * @retval None 
  */
void ADC_DisableMonitor(ADC_CMPCRx CMPCRx)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_CMPCRx(CMPCRx));

    if (CMPCRx == ADC_CMPCR_0) {
        /* read CMPCR0 then only keep bit4 to bit0, bit11 to bit8 */
        tmp = TSB_AD->CMPCR0 & ((ADC_MASK_KEEP5BITS << 0U) | (ADC_MASK_KEEP4BITS << 8U));
        TSB_AD->CMPCR0 = tmp;
    } else {
        /* read CMPCR1 then only keep bit4 to bit0, bit11 to bit8 */
        tmp = TSB_AD->CMPCR1 & ((ADC_MASK_KEEP5BITS << 0U) | (ADC_MASK_KEEP4BITS << 8U));
        TSB_AD->CMPCR1 = tmp;
    }
}


/**
  * @brief  Read ADC conversion result.
  * @param  ResultREGx: Select which ADC result register will be read.
  *   This parameter can be one of the following values:
  *     ADC_REG0, ADC_REG1, ADC_REG2, ADC_REG3, ADC_REG4, ADC_REG5,
  *     ADC_REG6, ADC_REG7, ADC_REG8, ADC_REG9, ADC_REG10, ADC_REG11  
  * @retval A union with AD result and 2 bits extra information. 
  *         Refer the comment for members of ADC_Result for more detail usage.
  */
ADC_Result ADC_GetConvertResult(ADC_REGx ResultREGx)
{
    ADC_Result result = { 0U };
    ADC_Resolution res = ADC_10BITS;

    /* Check the parameters */
    assert_param(IS_ADC_REGx(ResultREGx));

    result.All = *(ADC_REGx_Address[ResultREGx]);       /* read out result register wanted */
    result.All &= 0x0000FFFFU;  /* only keep bit15 to bit0 */

    res = ADC_GetResolution();
    if (res == ADC_10BITS) {
        result.Bit.ADResult >>= 2U;     /* 10bit mode: use bit15 to bit6 */
    } else {
        /* 12bit mode, use bit15 to bit4: Do nothing */
    }

    return result;
}



/**
  * @brief  Select PMD Trigger Program Number to be started by each of trigger inputs(PMD0 to PMD3).
  * @param  SELx : Specify the "trigger program number select register"
  *   This parameter can be one of the following values:  
  *     PMD_TRG_PROG_SEL0, PMD_TRG_PROG_SEL1, 
  *     PMD_TRG_PROG_SEL2, PMD_TRG_PROG_SEL3
  * @param  MacroProgNum : Program number to be selected, together with its Enabled or Disabled setting through a macro
  *   This parameter must be inputed with macro as the format below:
  *     TRG_ENABLE(x), TRG_DISABLE(x).
  *   'x' above can be one of the following values:
  *     PMD_PROG0, PMD_PROG1, PMD_PROG2
  *     PMD_PROG3, PMD_PROG4, PMD_PROG5 
  * @retval None
  */
void ADC_SelectPMDTrgProgNum(PMD_TRG_PROG_SELx SELx, uint8_t MacroProgNum)
{

    /* Check the parameters */
    assert_param(IS_PMD_TRG_PROG_SELx(SELx));
    assert_param(IS_PMD_PROGx_ENABLE_DISABLE(MacroProgNum));

    /* currently PSELx is defined as uint32_t */
    *(ADC_PSELx_Address[SELx]) = (uint32_t) MacroProgNum;
}



/**
  * @brief  Set the interrupt to be generated for all of PMD Trigger Program Numbers 
  * @param  TrgProgINT: The structure containing interrupt configuration for all of PMD Trigger Program Numbers
  *         Refer the comment for members of PMD_TrgProgINTTypeDef for more detail usage.
  * @retval None
  */
void ADC_SetPMDTrgProgINT(PMD_TrgProgINTTypeDef * TrgProgINT)
{
    /* Check the parameters */
    assert_param(IS_POINTER_NOT_NULL(TrgProgINT));
    assert_param(IS_PMD_INT_NAME(TrgProgINT->INTProg0));
    assert_param(IS_PMD_INT_NAME(TrgProgINT->INTProg1));
    assert_param(IS_PMD_INT_NAME(TrgProgINT->INTProg2));
    assert_param(IS_PMD_INT_NAME(TrgProgINT->INTProg3));
    assert_param(IS_PMD_INT_NAME(TrgProgINT->INTProg4));
    assert_param(IS_PMD_INT_NAME(TrgProgINT->INTProg5));

    /* currently PINTSx is defined as uint32_t */
    TSB_AD->PINTS0 = (uint32_t) TrgProgINT->INTProg0;
    TSB_AD->PINTS1 = (uint32_t) TrgProgINT->INTProg1;
    TSB_AD->PINTS2 = (uint32_t) TrgProgINT->INTProg2;
    TSB_AD->PINTS3 = (uint32_t) TrgProgINT->INTProg3;
    TSB_AD->PINTS4 = (uint32_t) TrgProgINT->INTProg4;
    TSB_AD->PINTS5 = (uint32_t) TrgProgINT->INTProg5;
}



/**
  * @brief  Set PMD Trigger Program Register: ADPSETx, where x = 0 to 5
  * @param  PMDTrg: The structure containing configuration for all of PMD Trigger Program Register 
  *         Refer the members of PMD_TrgTypeDef in .h file for more detail usage. 
  * @retval None
  */
void ADC_SetPMDTrg(PMD_TrgTypeDef * PMDTrg)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_POINTER_NOT_NULL(PMDTrg));
    assert_param(IS_PMD_PROGx(PMDTrg->ProgNum));
    assert_param(IS_ADC_AINx(PMDTrg->Reg0_AINx));
    assert_param(IS_ADC_AINx(PMDTrg->Reg1_AINx));
    assert_param(IS_ADC_AINx(PMDTrg->Reg2_AINx));
    assert_param(IS_ADC_AINx(PMDTrg->Reg3_AINx));

    tmp = (uint32_t) (PMDTrg->Reg0_AINx);
    tmp |= ((uint32_t) (PMDTrg->Reg1_AINx) << 8U);
    tmp |= ((uint32_t) (PMDTrg->Reg2_AINx) << 16U);
    tmp |= ((uint32_t) (PMDTrg->Reg3_AINx) << 24U);

    *ADC_PSETx_Address[PMDTrg->ProgNum] = tmp;
}



/**
  * @brief  Set Timer Trigger Program Register: TSETx, where x = 0 to 11
  * @param  ResultREGx: Select which ADC result register will be used for the specified ADC channel.
  *   This parameter can be one of the following values(12 registers):
  *     ADC_REG0, ADC_REG1, ADC_REG2, ADC_REG3, ADC_REG4, ADC_REG5,
  *     ADC_REG6, ADC_REG7, ADC_REG8, ADC_REG9, ADC_REG10, ADC_REG11  
  * @param  MacroAINx: Select AD Channel together with its enabled or disabled setting.
  *   This parameter must be inputed with macro as the format below:
  *     TRG_ENABLE(x), TRG_DISABLE(x).
  *   'x' above can be one of the following values:
  *     ADC_AIN0, ADC_AIN1, ADC_AIN2, ADC_AIN3, ADC_AIN4, ADC_AIN5, 
  *     ADC_AIN6, ADC_AIN7, ADC_AIN8, ADC_AIN9, ADC_AIN10, ADC_AIN11, 
  *     ADC_AIN12, ADC_AIN13, ADC_AIN14, ADC_AIN15, ADC_AIN16, 
  *     ADC_AIN17, ADC_AIN18, ADC_AIN19, ADC_AIN20, ADC_AIN21
  * @retval None
  */
void ADC_SetTimerTrg(ADC_REGx ResultREGx, uint8_t MacroAINx)
{
    uint32_t idx = 0U;
    uint32_t shift_byte = 0U;
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_REGx(ResultREGx));
    assert_param(IS_ADC_AINx(MacroAINx));

    idx = (uint32_t) (ResultREGx) / 4U; /* to index 03, 47, 811 */
    shift_byte = ((uint32_t) (ResultREGx) % 4U);        /* target shift byte: 0,1,2,3 */

    tmp = *ADC_TSETx_Address[idx];
    tmp &= ADC_ByteClear_Mask[shift_byte];      /* clear target area */
    tmp |= (uint32_t) (MacroAINx) << (shift_byte * 8U); /* target shift bits: 0, 8, 16, 24 */
    *ADC_TSETx_Address[idx] = tmp;
}


/**
  * @brief  Set Software Trigger Program Register: SSETx, where x = 0 to 11 
  * @param  ResultREGx: Select which ADC result register will be used for the specified ADC channel.
  *   This parameter can be one of the following values(12 registers):
  *     ADC_REG0, ADC_REG1, ADC_REG2, ADC_REG3, ADC_REG4, ADC_REG5,
  *     ADC_REG6, ADC_REG7, ADC_REG8, ADC_REG9, ADC_REG10, ADC_REG11
  * @param  MacroAINx: Select AD Channel together with its enabled or disabled setting.
  *   This parameter must be inputed with macro as the format below:
  *     TRG_ENABLE(x), TRG_DISABLE(x).
  *   'x' above can be one of the following values:
  *     ADC_AIN0, ADC_AIN1, ADC_AIN2, ADC_AIN3, ADC_AIN4, ADC_AIN5, 
  *     ADC_AIN6, ADC_AIN7, ADC_AIN8, ADC_AIN9, ADC_AIN10, ADC_AIN11, 
  *     ADC_AIN12, ADC_AIN13, ADC_AIN14, ADC_AIN15, ADC_AIN16, 
  *     ADC_AIN17, ADC_AIN18, ADC_AIN19, ADC_AIN20, ADC_AIN21
  * @retval None
  */
void ADC_SetSWTrg(ADC_REGx ResultREGx, uint8_t MacroAINx)
{
    uint32_t idx = 0U;
    uint32_t shift_byte = 0U;
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_REGx(ResultREGx));
    assert_param(IS_ADC_AINx(MacroAINx));

    idx = (uint32_t) (ResultREGx) / 4U; /* to index 03, 47, 811 */
    shift_byte = ((uint32_t) (ResultREGx) % 4U);        /* target shift byte: 0,1,2,3 */

    tmp = *ADC_SSETx_Address[idx];
    tmp &= ADC_ByteClear_Mask[shift_byte];      /* clear target area */
    tmp |= (uint32_t) (MacroAINx) << (shift_byte * 8U); /* target shift bits: 0, 8, 16, 24 */
    *ADC_SSETx_Address[idx] = tmp;
}

/**
  * @brief  Set Constant Trigger Program Register: ASETx, where x = 0 to 11 
  * @param  ResultREGx: Select which ADC result register will be used for the specified ADC channel.
  *   This parameter can be one of the following values(12 registers):
  *     ADC_REG0, ADC_REG1, ADC_REG2, ADC_REG3, ADC_REG4, ADC_REG5,
  *     ADC_REG6, ADC_REG7, ADC_REG8, ADC_REG9, ADC_REG10, ADC_REG11
  * @param  MacroAINx: Select AD Channel together with its enabled or disabled setting.
  *   This parameter must be inputed with macro as the format below:
  *     TRG_ENABLE(x), TRG_DISABLE(x).
  *   'x' above can be one of the following values:
  *     ADC_AIN0, ADC_AIN1, ADC_AIN2, ADC_AIN3, ADC_AIN4, ADC_AIN5, 
  *     ADC_AIN6, ADC_AIN7, ADC_AIN8, ADC_AIN9, ADC_AIN10, ADC_AIN11, 
  *     ADC_AIN12, ADC_AIN13, ADC_AIN14, ADC_AIN15, ADC_AIN16, 
  *     ADC_AIN17, ADC_AIN18, ADC_AIN19, ADC_AIN20, ADC_AIN21
  * @retval None
  */
void ADC_SetConstantTrg(ADC_REGx ResultREGx, uint8_t MacroAINx)
{
    uint32_t idx = 0U;
    uint32_t shift_byte = 0U;
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_REGx(ResultREGx));
    assert_param(IS_ADC_AINx(MacroAINx));

    idx = (uint32_t) (ResultREGx) / 4U; /* to index 03, 47, 811 */
    shift_byte = ((uint32_t) (ResultREGx) % 4U);        /* target shift byte: 0,1,2,3 */

    tmp = *ADC_ASETx_Address[idx];
    tmp &= ADC_ByteClear_Mask[shift_byte];      /* clear target area */
    tmp |= (uint32_t) (MacroAINx) << (shift_byte * 8U); /* target shift bits: 0, 8, 16, 24 */
    *ADC_ASETx_Address[idx] = tmp;
}

/** @} */
/* End of group ADC_Exported_Functions */

/** @} */
/* End of group ADC */

/** @} */
/* End of group TX03_Periph_Driver */

#endif                          /* defined(__TMPM384_ADC_H) */
