/**
 *******************************************************************************
 * @file    tmpm384_ofd.c
 * @brief   This file provides API functions for OFD driver.
 * @version V2.0.2.2
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm384_ofd.h"

#if defined(__TMPM384_OFD_H)

/** @addtogroup TX03_Periph_Driver
  * @{
  */
/** @defgroup OFD
  * @brief OFD driver modules
  * @{
  */

/** @defgroup OFD_Private_Defines
  * @{
  */
#define OFD_REG_WRITE_DISABLE       ((uint32_t)0x00000006U)
#define OFD_REG_WRITE_ENABLE        ((uint32_t)0x000000F9U)
#define OFD_DISABLE                 ((uint32_t)0x00000000U)
#define OFD_ENABLE                  ((uint32_t)0x000000E4U)
/** @} */
/* End of group OFD_Private_Defines */

/** @defgroup OFD_Exported_Functions
  * @{
  */

/**
  * @brief  Enable or disable the writing of OFDCR2/OFDMN/OFDMX.
  * @param  NewState: the new state of enable or disable
  *             This parameter can be one of the following values:
  *             ENABLE or DISABLE
  * @retval None
  */
void OFD_SetRegWriteMode(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    /* Set OFDCR1<OFDWEN0-7> to enable or disable the writing of OFDCR2/OFDMN/OFDMX. */
    if (NewState == ENABLE) {
        TSB_OFD->CR1 = OFD_REG_WRITE_ENABLE;
    } else {
        TSB_OFD->CR1 = OFD_REG_WRITE_DISABLE;
    }
}

/**
  * @brief  Enable the OFD function.
  * @param  None
  * @retval None
  */
void OFD_Enable(void)
{
    /* OFDCR2<OFDEN0-7> = 0xE4 */
    TSB_OFD->CR2 = OFD_ENABLE;
}

/**
  * @brief  Disable the OFD function.
  * @param  None
  * @retval None
  */
void OFD_Disable(void)
{
    /* OFDCR2<OFDEN0-7> = 0x00 */
    TSB_OFD->CR2 = OFD_DISABLE;
}

/**
  * @brief  Set the count value of detection frequency.
  * @param  HigherDetectionCount: the count value of higher detection frequency.
  *         LowerDetectionCount: the count value of lower detection frequency.
  * @retval None
  */
void OFD_SetDetectionFrequency(uint8_t HigherDetectionCount, uint8_t LowerDetectionCount)
{
    /* Set the OFDMN<OFDMN0-7> and OFDMX<OFDMX0-7> bits */
    TSB_OFD->MN = (uint32_t) LowerDetectionCount;
    TSB_OFD->MX = (uint32_t) HigherDetectionCount;
}

/**
  * @brief  Enable or disable the OFD reset.
  * @param  NewState: the new state of enable or disable
  *             This parameter can be one of the following values:
  *             ENABLE or DISABLE
  * @retval None
  */
void OFD_Reset(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    /* Set OFDRST<OFDRSTEN> to enable or disable the OFD reset. */
    if (NewState == ENABLE) {
        TSB_OFD_RST_OFDRSTEN = 1U;
    } else {
        TSB_OFD_RST_OFDRSTEN = 0U;
    }
}

/**
  * @brief  Get the OFD busy and frequency error info.
  * @param  None
  * @retval The structure includes the OFD busy and frequency error status.
  */
OFD_Status OFD_GetStatus(void)
{
    OFD_Status status = { 0U };

    /*Read the ofd status from OFDSTAT<OFDBUSY><FRQEER> register. */
    status.All = TSB_OFD->STAT;
    return status;
}

/** @} */
/* End of group OFD_Exported_Functions */
/** @} */
/* End of group OFD */
/** @} */
/* End of group TX03_Periph_Driver */
#endif                          /* (__TMPM384_OFD_H) */
