/**
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of IGBT One PPG for the TOSHIBA 'TMPM384' 
 *          Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/*-------------------------
PPG wave is generated from MTOUT00.
1.	Use a wire to connect port GEMG0 with DVDD to pull it high.
2.	Connect MTOUT00 to oscilloscope.
3.	Power on, LED0 on and no waveform is observed from MTOUT00.
4.	Touch MT0IN with GND to produce a falling edge and keep it to low.
5.	The IGBT timer starts to run because of the falling edge and at the same time PPG waveform can be observed from MTOUT00. PPG cycle is 50us with duty 50%.
6.	Connect GEMG0 with GND, then PPG output stops immediately and LED1 lights on.
7.	Connect GEMG0 with DVDD again, and EMG state cancels with LED1 off, then repeat step 4 to observe the waveform.
------------------------- */

/*-------------------------
For M384, pins for IGBT0 as below:
  Function	     Pin Name	   Pin No.
 PPG output	    MTOUT00/PC4	    17
 PPG output 1	MTOUT10/PC5	    16
 EMG input	    GEMG0/PC6	    15
 Trigger input	MT0IN/PC7	    14
------------------------- */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* define IGBT channel used here */
/* remember to modify pin setting in IGBT_IO_Configuration() */
#define IGBT_CHANNEL    IGBT0
#define IGBT_EMG_INT    INTMTEMG0_IRQn


/* Global variable define ----------------------------------------------------*/
WorkState counter_state = BUSY;
Result cancel_result = ERROR;
IGBT_InitTypeDef myIGBT;
IGBT_EMGStateTypeDef EMGState = { IGBT_EMG_INPUT_LOW, IGBT_EMG_NORMAL };



/**
  * @brief  main function to run IGBT One PPG demo
  * @param  None.
  * @retval result to retun to system.
  */
int main(void)
{
    /* LED initialization */
    LED_Init();
    LED_On(LED0);

    myIGBT.StartMode = IGBT_FALLING_TRG_START;          /* IGBT trigger start: falling edge start and active level is "Low" */
    myIGBT.OperationMode = IGBT_CONTINUOUS_OUTPUT;      /* IGBT operation: continuous operation */
    myIGBT.CntStopState = IGBT_OUTPUT_INACTIVE;         /* IGBT stopping status: initial output satus and counter */
    myIGBT.ActiveAcceptTrg = DISABLE;                   /* Trigger edge accept mode: Don't accept trigger during active level */
    myIGBT.INTPeriod = IGBT_INT_PERIOD_1;               /* Interrupt cycle: Every one cycle */
    myIGBT.ClkDiv = IGBT_CLK_DIV_2;                     /* fperiph = fc = 10MHz*4 = 40MHz, T0 = fperiph = 40MHz, figbt = T0/2 = 20MHz */
    myIGBT.Output0Init = IGBT_OUTPUT_HIGH_ACTIVE;       /* MTOUT00 initial state is Low, and active level is High */
    myIGBT.Output1Init = IGBT_OUTPUT_DISABLE;           /* Disable MTOUT10 output */
    myIGBT.TrgDenoiseDiv = IGBT_DENOISE_DIV_240;        /* Trigger input noise elimination time: 240/fsys */
    myIGBT.Output0ActiveTiming = 1U;
    myIGBT.Output0InactiveTiming = 1U + IGBT_PPG_PERIOD_50US / 2;
    myIGBT.Period = IGBT_PPG_PERIOD_50US;               /* Period: 50us */
    myIGBT.EMGFunction = IGBT_EMG_OUTPUT_HIZ;           /* The polarity of MTOUT0x at EMG protection: High-impedance */
    myIGBT.EMGDenoiseDiv = IGBT_DENOISE_DIV_240;        /* EMG input noise elimination time: 240/fsys */

    IGBT_IO_Configuration(IGBT_CHANNEL, &myIGBT);

    IGBT_Enable(IGBT_CHANNEL);
    NVIC_EnableIRQ(INTMTEMG0_IRQn);                     /* Enable IGBT and EMG interrupt */

    do {                                                /* If the timer is still running, wait until it stops */
        counter_state = IGBT_GetCntState(IGBT_CHANNEL);
    } while (counter_state == BUSY);
    do {                                                /* Cancel the EMG protection state */
        cancel_result = IGBT_CancelEMGState(IGBT_CHANNEL);
    } while (cancel_result == ERROR);

    IGBT_Init(IGBT_CHANNEL, &myIGBT);

    IGBT_SetSWRunState(IGBT_CHANNEL, IGBT_RUN);                /* Prepare to run IGBT timer */

    while (1) {

        EMGState = IGBT_GetEMGState(IGBT_CHANNEL);             /* Get EMG protection state to check EMG state */

        if (EMGState.EMGProtectState == IGBT_EMG_PROTECT) {     /* If EMG protection is on going */
            do {
                counter_state = IGBT_GetCntState(IGBT_CHANNEL);
            } while (counter_state == BUSY);
            do {                                        /* Cancel the EMG protection state */
                cancel_result = IGBT_CancelEMGState(IGBT_CHANNEL);
            } while (cancel_result == ERROR);

            LED_Off(LED1);                              /* Turn off the LED and prepare to run again */
            IGBT_Init(IGBT_CHANNEL, &myIGBT);

            IGBT_SetSWRunState(IGBT_CHANNEL, IGBT_RUN);
        } else {
            /* Do nothing */
        }
    }
}

/**
  * @brief  IRQ for EMG protection
  * @param  None.
  * @retval None.
  */
void INTMTEMG0_IRQHandler(void)
{
    LED_On(LED1);                                       /* If EMG protection, turn on the LED and stop the timer */
    IGBT_SetSWRunState(IGBT_CHANNEL, IGBT_STOP);
}

/**
  * @brief  IGBT port initialization
  * @param  IGBTx: IGBT channel.
  *         InitTypeDef: initialization structure.
  * @retval None.
  */
void IGBT_IO_Configuration(TSB_MT_TypeDef * IGBTx, IGBT_InitTypeDef * InitTypeDef)
{
    if (IGBTx == IGBT_CHANNEL) {
        /* Check whether the trigger is used to start */
        if ((InitTypeDef->StartMode != IGBT_CMD_START) ||
            (InitTypeDef->StartMode != IGBT_CMD_START_NO_START_INT)) {
            GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_7, ENABLE);        /* Enable PC7 input */
            GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_7);   /* Set PC7 as MT0IN */
        } else {
            GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_7, DISABLE);
            GPIO_DisableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_7);
        }
        /* Enable ouput0 or not */
        if (InitTypeDef->Output0Init != IGBT_OUTPUT_DISABLE) {
            GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_4, ENABLE);       /* Enable PC4 output */
            GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_4);   /* Set PC4 as MTOUT00 */
        } else {
            GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_4, DISABLE);
            GPIO_DisableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_4);
        }
        /* Enable ouput1 or not */ 
        if (InitTypeDef->Output1Init != IGBT_OUTPUT_DISABLE) {
            GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_5, ENABLE);       /* Enable PC5 output */
            GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_5);   /* Set PC5 as MTOUT10 */
        } else {
            GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_5, DISABLE);
            GPIO_DisableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_5);
        }

        GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_6, ENABLE);    /* Enable PC6 input */
        GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_6);       /* Set PC6 as #GEMG0 */
    }

}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif                          /* DEBUG */
