/**
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of IGBT Two PPG for the TOSHIBA 'TMPM384' 
 *          Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/*-------------------------
Two PPG waves are generated from MTOUT00 and MTOUT10.
1.	Use a wire to connect port GEMG0 with DVDD to pull it high.
2.	Connect MTOUT00 and MTOUT10 to oscilloscope.
3.	Switch SW7 to OFF.
4.	Power on, LED0 on and no waveforms are observed from MTOUT00 or MTOUT10.
5.	Switch SW7 to ON.
6.	The IGBT timer starts to run and at the same time PPG waveform can be observed from MTOUT00 and MTOUT10. Both PPG waveforms cycle is 50us with duty 40 %( high level: 20us) and MTOUT10 delays MTOUT00 with 25us.
7.	If switch SW7 to OFF, PPG waveform disappears.
8.	Connect GEMG0 with GND, then PPG output stops immediately and LED1 lights on.
9.	Connect GEMG0 with DVDD again, and EMG state cancels with LED1 off, then repeat step 5 to observe the waveform.
------------------------- */

/*-------------------------
For M384, pins for IGBT_CHANNEL as below:
  Function	     Pin Name	   Pin No.
 PPG output	    MTOUT00/PC4	    17
 PPG output 1	MTOUT10/PC5	    16
 EMG input	    GEMG0/PC6	    15
 Trigger input	MT0IN/PC7	    14
------------------------- */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* define IGBT channel used here */
/* remember to modify pin setting in IGBT_IO_Configuration() */
#define IGBT_CHANNEL    IGBT0
#define IGBT_EMG_INT    INTMTEMG0_IRQn


/* Global variable define ----------------------------------------------------*/
WorkState counter_state = BUSY;
Result cancel_result = ERROR;
IGBT_InitTypeDef myIGBT;
IGBT_EMGStateTypeDef EMGState = { IGBT_EMG_INPUT_LOW, IGBT_EMG_NORMAL };

int main(void)
{
    /* LED & swich initialization */
    LED_Init();
    SW_Init();
    LED_On(LED0);

    myIGBT.StartMode = IGBT_CMD_START;                  /* IGBT trigger start: command start */
    myIGBT.OperationMode = IGBT_CONTINUOUS_OUTPUT;      /* IGBT operation: continuous operation */
    myIGBT.CntStopState = IGBT_OUTPUT_INACTIVE;         /* IGBT stopping status: initial output satus and counter */
    myIGBT.ActiveAcceptTrg = DISABLE;                   /* Trigger edge accept mode: Don't accept trigger during active level */
    myIGBT.INTPeriod = IGBT_INT_PERIOD_1;               /* Interrupt cycle: Every one cycle */
    myIGBT.ClkDiv = IGBT_CLK_DIV_2;                     /* fperiph = fc = 10MHz*4 = 50MHz, T0 = fperiph = 40MHz, figbt = T0/2 = 20MHz */
    myIGBT.Output0Init = IGBT_OUTPUT_HIGH_ACTIVE;       /* MTOUT00 initial state is Low, and active level is High */
    myIGBT.Output1Init = IGBT_OUTPUT_HIGH_ACTIVE;       /* MTOUT10 initial state is Low, and active level is High */
    myIGBT.TrgDenoiseDiv = IGBT_NO_DENOISE;             /* Trigger input noise elimination time: no use */
    myIGBT.Output0ActiveTiming = 1U;
    myIGBT.Output0InactiveTiming = 1U + IGBT_ACTIVE_PERIOD_20US;
    myIGBT.Output1ActiveTiming = myIGBT.Output0InactiveTiming + IGBT_DEAD_TIME_5US;
    myIGBT.Output1InactiveTiming = myIGBT.Output1ActiveTiming + IGBT_ACTIVE_PERIOD_20US;
    myIGBT.Period = IGBT_PPG_PERIOD_50US;               /* Period: 50us */
    myIGBT.EMGFunction = IGBT_EMG_OUTPUT_HIZ;           /* The polarity of MTOUT0x at EMG protection: High-impedance */
    myIGBT.EMGDenoiseDiv = IGBT_DENOISE_DIV_240;        /* EMG input noise elimination time: 240/fsys */

    IGBT_IO_Configuration(IGBT_CHANNEL, &myIGBT);

    IGBT_Enable(IGBT_CHANNEL);
    NVIC_EnableIRQ(IGBT_EMG_INT);                     /* Enable IGBT and EMG interrupt */

    do {                                                /* If the timer is still running, wait until it stops */
        counter_state = IGBT_GetCntState(IGBT_CHANNEL);
    } while (counter_state == BUSY);
    do {                                                /* Cancel the EMG protection state */
        cancel_result = IGBT_CancelEMGState(IGBT_CHANNEL);
    } while (cancel_result == ERROR);

    IGBT_Init(IGBT_CHANNEL, &myIGBT);

    if (SWITCH_ON) {                                    /* If switch is ON */
        IGBT_SetSWRunState(IGBT_CHANNEL, IGBT_RUN);            /* Start to run IGBT timer */
    } else {
        /* Do nothing */
    }

    while (1) {

        EMGState = IGBT_GetEMGState(IGBT_CHANNEL);             /* Get EMG protection state to check EMG state */

        if (EMGState.EMGProtectState == IGBT_EMG_PROTECT) {     /* If EMG protection is on going */
            do {
                counter_state = IGBT_GetCntState(IGBT_CHANNEL);
            } while (counter_state == BUSY);
            do {                                        /* Cancel the EMG protection state */
                cancel_result = IGBT_CancelEMGState(IGBT_CHANNEL);
            } while (cancel_result == ERROR);

            LED_Off(LED1);                              /* Turn off the LED and start to run again */
            IGBT_Init(IGBT_CHANNEL, &myIGBT);

            IGBT_SetSWRunState(IGBT_CHANNEL, IGBT_RUN);

        } else {
            /* Do nothing */
        }

        if (SWITCH_ON) {
            IGBT_SetSWRunState(IGBT_CHANNEL, IGBT_RUN);        /* If switch is ON, start to run */
        } else {
            IGBT_SetSWRunState(IGBT_CHANNEL, IGBT_STOP);       /* If switch is OFF, stop to run */
        }
    }
}

/**
  * @brief  IRQ for EMG protection
  * @param  None.
  * @retval None.
  */
void INTMTEMG0_IRQHandler(void)
{
    LED_On(LED1);                                       /* If EMG protection, turn off the LED and stop the timer */
    IGBT_SetSWRunState(IGBT_CHANNEL, IGBT_STOP);
}

/**
  * @brief  IGBT port initialization
  * @param  IGBTx: IGBT channel.
  *         InitTypeDef: initialization structure.
  * @retval None.
  */
void IGBT_IO_Configuration(TSB_MT_TypeDef * IGBTx, IGBT_InitTypeDef * InitTypeDef)
{
    if (IGBTx == IGBT_CHANNEL) {
        /* Check whether the trigger is used to start */
        if ((InitTypeDef->StartMode != IGBT_CMD_START) ||
            (InitTypeDef->StartMode != IGBT_CMD_START_NO_START_INT)) {
            GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_7, ENABLE);        /* Enable PC7 input */
            GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_7);   /* Set PC7 as MT0IN */
        } else {
            GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_7, DISABLE);
            GPIO_DisableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_7);
        }
        /* Enable ouput0 or not */
        if (InitTypeDef->Output0Init != IGBT_OUTPUT_DISABLE) {
            GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_4, ENABLE);       /* Enable PC4 output */
            GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_4);   /* Set PC4 as MTOUT00 */
        } else {
            GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_4, DISABLE);
            GPIO_DisableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_4);
        }
        /* Enable ouput1 or not */ 
        if (InitTypeDef->Output1Init != IGBT_OUTPUT_DISABLE) {
            GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_5, ENABLE);       /* Enable PC5 output */
            GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_5);   /* Set PC5 as MTOUT10 */
        } else {
            GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_5, DISABLE);
            GPIO_DisableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_5);
        }

        GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_6, ENABLE);    /* Enable PC6 input */
        GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_6);       /* Set PC6 as #GEMG0 */
    }
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif                          /* DEBUG */
