/**
 *******************************************************************************
 * @file    dma.c
 * @brief   Code file to initialize DMA for application "SSP0 LoopBack with DMA 
 *          in Tx/Rx" for the TOSHIBA 'TMPM384' Device Series
 * @brief   The demo application of "SSP0 LoopBack with DMA in Tx/Rx" for the
 *          TOSHIBA 'TMPM384' Device Series
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/* Include header file */
#include "tmpm384_dmac_int.h"
#include "dma.h"

/* User ram definition */
const uint16_t SRC_Buffer[BUFFER_SIZE] = {
    0x1111U, 0x1111U, 0x1111U, 0x1111U,
    0x2222U, 0x2222U, 0x2222U, 0x2222U,    
    0x3333U, 0x3333U, 0x3333U, 0x3333U,
    0x4444U, 0x4444U, 0x4444U, 0x4444U, 
    0x5555U, 0x5555U, 0x5555U, 0x5555U,
    0x6666U, 0x6666U, 0x6666U, 0x6666U, 
    0x7777U, 0x7777U, 0x7777U, 0x7777U,
    0x8888U, 0x8888U, 0x8888U, 0x8888U, 
    0x9999U, 0x9999U, 0x9999U, 0x9999U,
    0xAAAAU, 0xAAAAU, 0xAAAAU, 0xAAAAU, 
    0xBBBBU, 0xBBBBU, 0xBBBBU, 0xBBBBU,
    0xCCCCU, 0xCCCCU, 0xCCCCU, 0xCCCCU,
    0xDDDDU, 0xDDDDU, 0xDDDDU, 0xDDDDU,
    0xEEEEU, 0xEEEEU, 0xEEEEU, 0xEEEEU,
    0xFFFFU, 0xFFFFU, 0xFFFFU, 0xFFFFU                    
};

uint16_t DST_Buffer[BUFFER_SIZE] = { 0U };


static void InitDMA0(void);
static void InitDMA1(void);

/**
  * @brief  initialize DMA0 and DMA1
  * @param  none
  * @retval None
  */
void InitDMA(void)
{
    InitDMA0(); 
    InitDMA1();  
}


/**
  * @brief  initialize DMA0 as Memory to SSP0_Tx
  * @param  None
  * @retval None
  */
void InitDMA0(void)
{
    DMAC_InitTypeDef DMAC_InitStruct;

    DMAC_InitStruct.SrcAddr = (uint32_t) SRC_Buffer;
    DMAC_InitStruct.SrcIncrementState = ENABLE;
    DMAC_InitStruct.SrcBitWidth = DMAC_HALF_WORD;   /* 16bit */
    DMAC_InitStruct.SrcBurstSize = DMAC_4_BEATS;
    
    DMAC_InitStruct.DstAddr = (uint32_t) (&TSB_SSP0->DR) ;
    DMAC_InitStruct.DstIncrementState = DISABLE;
    DMAC_InitStruct.DstBitWidth = DMAC_HALF_WORD;
    DMAC_InitStruct.DstBurstSize = DMAC_4_BEATS;
    
    DMAC_InitStruct.TxSize = BUFFER_SIZE;
    DMAC_InitStruct.TxDirection = DMAC_MEMORY_TO_PERIPH ;
    
    DMAC_InitStruct.TxPeriph = DMAC_SSP0_TX;
    DMAC_InitStruct.TxINT = DISABLE;

    __disable_irq();

    /*Enable DMA circuit */
    DMAC_Enable();
    
    /*Initialize DMA channel 0 */
    DMAC_Init(DMAC_CHANNEL_0, &DMAC_InitStruct);
    
    /* If don't require scatter/gather function, set the DMACCxLLI Register to 0 */
    DMAC_SetLinkedList(DMAC_CHANNEL_0, 0U);
        
    /*Enable DMA channel */
    DMAC_SetDMAChannel(DMAC_CHANNEL_0, ENABLE);
    
    __enable_irq();

}


/**
  * @brief  initialize DMA1 as SSP1_Rx to Memory
  * @param  None
  * @retval None
  */
void InitDMA1(void)
{
    DMAC_InitTypeDef DMAC_InitStruct;

    DMAC_InitStruct.SrcAddr = (uint32_t) (&TSB_SSP1->DR);
    DMAC_InitStruct.SrcIncrementState = DISABLE;
    DMAC_InitStruct.SrcBitWidth = DMAC_HALF_WORD;   /* 16bit */
    DMAC_InitStruct.SrcBurstSize = DMAC_4_BEATS;
    
    DMAC_InitStruct.DstAddr = (uint32_t) DST_Buffer ;
    DMAC_InitStruct.DstIncrementState = ENABLE;
    DMAC_InitStruct.DstBitWidth = DMAC_HALF_WORD;
    DMAC_InitStruct.DstBurstSize = DMAC_4_BEATS;
    
    DMAC_InitStruct.TxSize = BUFFER_SIZE;
    DMAC_InitStruct.TxDirection = DMAC_PERIPH_TO_MEMORY ;
    
    DMAC_InitStruct.TxPeriph = DMAC_SSP1_RX;
    

    /* CXCONTROL<I> must be set when use transfer interrupt */
    DMAC_InitStruct.TxINT = ENABLE;

    __disable_irq();
    
    /* enable DMA interrupt */
    NVIC_ClearPendingIRQ(INTDMACTC_IRQn);
    NVIC_EnableIRQ(INTDMACTC_IRQn);
    
    /*Enable DMA Channel transfer end interrupt */
    DMAC_SetTxINTConfig(DMAC_CHANNEL_1, DMAC_INT_TX_END, ENABLE); 
    
    __enable_irq();   
    /*Enable DMA circuit */
    DMAC_Enable();

    /*Initialize DMA channel */
    DMAC_Init(DMAC_CHANNEL_1, &DMAC_InitStruct);
    
    /* If don't require scatter/gather function, set the DMACCxLLI Register to 0 */
    DMAC_SetLinkedList(DMAC_CHANNEL_1, 0U);

    /*Enable DMA channel */
    DMAC_SetDMAChannel(DMAC_CHANNEL_1, ENABLE);
     
}
