/**
 *******************************************************************************
 * @file    tmpm384_dmac_int.c
 * @brief   test M384 SPI Host to Slave (SSP0 to SSP1)
 *           for the TOSHIBA 'TMPM384' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
 
/*-------------------------
For M384, there are 2 SSP channels. 
SSP0 is set as SPI Mast while SSP1 is set as SPI Slave, then connect correspond pins, 
 use DMA to transmit/receive infinite self increased data.
-------------------------*/



/* Includes ------------------------------------------------------------------*/
#include "tmpm384_ssp.h"
#include "tmpm384_gpio.h"
#include "dma.h"

typedef enum { SAME = 1U, NOT_SAME = !SAME } TestResult;

void DisplayLED(uint16_t dat);
void initSSP(void);
void GPIO_SetSSP(void);
TestResult Buffercompare(const uint16_t * Src, const uint16_t * Dst, uint32_t Len);

#define MAX_BUFSIZE  64U

TestResult result = NOT_SAME;

/* to check the effect due to too low speed, uncomment the line below */
/* #define BITRATE_MIN */



/*-------------------------- Pin Connection and Setting --------------------------------
	SSP0 pins	                Note	                        SSP1 pins
1	PC3/SP0FSS, pin18 	Connect PC3 to PN3(pin18 to 81)	    PN3/SP1FSS , pin81 
2	PC2/SP0CLK, pin19	Connect PC2 to PN2(pin19 to 67)	    PN2/SP1CLK , pin67
3	PC1/SP0DI, pin20	Connect PC1 to PN0(pin20 to 65)	    PN1/SP1DI,  pin66   
4	PC0/SP0DO, pin21	Connect PC0 to PN1(pin21 to 66)	    PN0/SP1DO, pin65   
 
 
//------------------------- 
CR.x: call GPIO_SetOutputEnableReg()
PC0F2: call: GPIO_EnableFuncReg()
IE.x: call: GPIO_SetInputEnableReg()
OD.x: call: GPIO_SetOpenDrain()
-------------------------------*/
void GPIO_SetSSP(void)
{
    /* PC0 to be used as SP0DO: */
    GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_0, ENABLE);
    GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_0);
    GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_0, DISABLE);

    /* PC1 to be used as SP0DI: */
    GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_1, ENABLE);
    GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_1);
    GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_1, DISABLE);
    GPIO_SetOpenDrain(GPIO_PC, GPIO_BIT_1, DISABLE);

    /* PC2 to be used as SP0CLK out: */
    GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_2, ENABLE);
    GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_2);
    GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_2, DISABLE);

    /* PC3 to be used as SP0FSS out: */
    GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_3, ENABLE);
    GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_3);
    GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_3, DISABLE); 



    /* PN0 to be used as SP1DO: */
    GPIO_SetOutputEnableReg(GPIO_PN, GPIO_BIT_0, ENABLE);
    GPIO_EnableFuncReg(GPIO_PN, GPIO_FUNC_REG_1, GPIO_BIT_0);
    GPIO_SetInputEnableReg(GPIO_PN, GPIO_BIT_0, DISABLE);

    /* PN1 to be used as SP1DI: */
    GPIO_SetInputEnableReg(GPIO_PN, GPIO_BIT_1, ENABLE);
    GPIO_EnableFuncReg(GPIO_PN, GPIO_FUNC_REG_1, GPIO_BIT_1);
    GPIO_SetOutputEnableReg(GPIO_PN, GPIO_BIT_1, DISABLE);
    GPIO_SetOpenDrain(GPIO_PN, GPIO_BIT_1, DISABLE);

    /* PN2 to be used as SP1CLK in: */
    GPIO_SetInputEnableReg(GPIO_PN, GPIO_BIT_2, ENABLE);
    GPIO_EnableFuncReg(GPIO_PN, GPIO_FUNC_REG_1, GPIO_BIT_2);
    GPIO_SetOutputEnableReg(GPIO_PN, GPIO_BIT_2, DISABLE);
    GPIO_SetOpenDrain(GPIO_PN, GPIO_BIT_2, DISABLE);

    /* PN3 to be used as SP1FSS in: */
    GPIO_SetInputEnableReg(GPIO_PN, GPIO_BIT_3, ENABLE);        
    GPIO_EnableFuncReg(GPIO_PN, GPIO_FUNC_REG_1, GPIO_BIT_3);  
    GPIO_SetOutputEnableReg(GPIO_PN, GPIO_BIT_3, DISABLE);     
    GPIO_SetOpenDrain(GPIO_PN, GPIO_BIT_3, DISABLE);  
    
}    

void initSSP(void)
{
    SSP_InitTypeDef initSSP;
        
    /* init SSP0 as SPI Host and SSP1 as SPI Slave */
    
    /* configure the SSP module */
    initSSP.FrameFormat = SSP_FORMAT_SPI;

    /* set bit rate */
    initSSP.PreScale = 8U;
    initSSP.ClkRate = 8U;
    /* define BITRATE_MIN to run at minimum bit rate */
    /*   BitRate = fSYS / (PreScale x (1 + ClkRate)) */
#ifdef BITRATE_MIN
    initSSP.PreScale = 254U;
    initSSP.ClkRate = 255U;
#endif
    initSSP.ClkPolarity = SSP_POLARITY_LOW;
    initSSP.ClkPhase = SSP_PHASE_FIRST_EDGE;
    initSSP.DataSize = 16U;
    initSSP.Mode = SSP_MASTER;
    SSP_Init(TSB_SSP0, &initSSP);
    
    initSSP.ClkPhase = SSP_PHASE_FIRST_EDGE;
    initSSP.Mode = SSP_SLAVE;
    SSP_Init(TSB_SSP1, &initSSP);

    /* enable DMA in SSP module*/
    SSP_SetDMACtrl(TSB_SSP0, SSP_TX, ENABLE);
    SSP_SetDMACtrl(TSB_SSP1, SSP_RX, ENABLE);
    
    /* enable and run SSP module */
    SSP_Enable(TSB_SSP0);
    SSP_Enable(TSB_SSP1);
}

int main(void)
{
    GPIO_SetSSP();
    initSSP();
    InitDMA();
    
    /* Wait the end of transmission */
    while (TxEndFlag != DONE) {
        /* Do nothing */
    }
    
    /* now DMA is finished, Set a Break Point here, */
    /* after function Buffercompare() is called, result == SAME */
    result = Buffercompare( SRC_Buffer, DST_Buffer, BUFFER_SIZE);
    
    while(1) {
        /* do nothing */
    }
}

/**
  * @brief  To compare two half word buffer
  * @param  Src : pointer to point source buffer 
  * @param  Dst : pointer to point destination buffer   
  * @param  Len : how many data will be compared 
  * @retval SAME: two buffer contain same data
            NOT_SAME: two buffer contain different data  
  */
TestResult Buffercompare(const uint16_t * Src, const uint16_t * Dst, uint32_t Len)
{
    uint32_t tmp = 0U;
    TestResult result = NOT_SAME;
    
    for( tmp = 0U; tmp < Len ; tmp++ ) {
        if( Src[tmp] != Dst[tmp] ) {
             break;
       } else {
        result = SAME;
       }
    }

    return result;
}    

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while(1){
        __NOP();
    }
}
#endif
