/**
 *******************************************************************************
 * @file    lcd.c
 * @brief   The driver of LCD for the
 *           TOSHIBA 'TMPM384' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Connection for following LCD driver 4BIT bus 
 * PJ4~7                    <----> PIN19~22 on PJ24  : LCDB4~B7                                  
 * PV2                      <----> PIN11    on PJ24  : LCDE
 * PV1                      <----> PIN12    on PJ24  : LCDRW
 * PV0                      <----> PIN13    on PJ24  : LCDRS
 */

/* Connect 3 pins on PJ24 to MCU
PJ24  <------> MCU
PIN11 <------> PIN3
PIN12 <------> PIN2
PIN13 <------> PIN1
*/

/* Includes ------------------------------------------------------------------*/
#include "lcd.h"
#include <string.h>
#include "tmpm384_gpio.h"

//select 4BIT bus or 8BIT bus
//#define BUS_4BIT

#define LCD_DATA            (TSB_PJ->DATA)
#define LCD_DATA_PORT        GPIO_PJ

#define PV_IS_I_PORT         //PV can't be output port
#ifdef  PV_IS_I_PORT

#define LCDE                (TSB_PU_DATA_PU2)
#define LCDRW               (TSB_PU_DATA_PU1)
#define LCDRS               (TSB_PU_DATA_PU0)
#define LCD_CTRL_PORT       GPIO_PU
#else
#define LCDE                (TSB_PV_DATA_PV2)
#define LCDRW               (TSB_PV_DATA_PV1)
#define LCDRS               (TSB_PV_DATA_PV0)
#define LCD_CTRL_PORT       GPIO_PV
#endif

#define LCDE_BIT            GPIO_BIT_2
#define LCDRW_BIT           GPIO_BIT_1
#define LCDRS_BIT           GPIO_BIT_0
#define LCD_WRITE           0UL
#define LCD_READ            1UL


void Delay_LCD(uint32_t time);

/*******************************************************************************
* Function Name  : E_Pulse
* Description    : Generation a pulse enable the LCD.
* Input          : None.
* Return         : None.
*******************************************************************************/
void E_Pulse(void)
{
    // set E high
    LCDE = 1;
    Delay_LCD(100);
    // set E low

    LCDE = 0;
    Delay_LCD(100);
}

/*******************************************************************************
* Function Name  : Set_Data
* Description    : transfer data to LCD
* Input          : data --- The data will transfer to LCD
* Return         : None.
*******************************************************************************/
void Set_Data(unsigned char data)
{
#ifdef BUS_4BIT
    uint8_t cTemp = 0U;
    cTemp = LCD_DATA & 0x0F;
    data = ((uint8_t) (data << 4)) | cTemp;
#endif
    LCD_DATA = data;
}

/*******************************************************************************
* Function Name  : Send_To_LCD
* Description    : send command or data to LCD
* Input          : data --- command or data
*                  type --- LCD_COMMAND or LCD_CHARACTER
* Return         : None.
*******************************************************************************/
void Send_To_LCD(unsigned char data, unsigned char type)
{
    // enable write
    
    LCDRW = LCD_WRITE;
    Delay_LCD(100);
#ifdef BUS_4BIT
    // set high nibble of command
    Set_Data((data >> 4));
#else
    Set_Data(data);
#endif
    // set RS port to set command mode or data
    if (type == LCD_COMMAND)
        LCDRS = 0;
    else
        LCDRS = 1;
    E_Pulse();
#ifdef BUS_4BIT
    // set low nibble of command
    Set_Data((data & 0x0F));
    // set RS port to set command mode or data
    if (type == LCD_COMMAND)
        LCDRS = 0;
    else
        LCDRS = 1;
    // pulse to set D0-D3 bits
    E_Pulse();
#endif
}

/*******************************************************************************
* Function Name  : Send_LCD_Text
* Description    : Send the string to LCD
* Input          : str --- pointer of the string
* Return         : None.
*******************************************************************************/
void Send_LCD_Text(char *str)
{
    unsigned char i;
    /* loop over string and send each character */
    for (i = 0; i < strlen(str); i++) {
        Send_To_LCD(str[i], LCD_CHARACTER);
    }
    for (; i < 16; i++)
        Send_To_LCD(' ', LCD_CHARACTER);
}

/*******************************************************************************
* Function Name  : LCD_IO
* Description    : Config LCD to IO
* Input          : data --- The data will transfer to LCD
* Return         : None.
*******************************************************************************/

void LCD_IO(void)
{
#ifdef BUS_4BIT
    GPIO_SetOutput(LCD_DATA_PORT,0xF0);
    GPIO_SetPullUp(LCD_DATA_PORT,0xF0,ENABLE);
#else
    GPIO_SetOutput(LCD_DATA_PORT,0xFF);
    GPIO_SetPullUp(LCD_DATA_PORT,0xFF,ENABLE);
#endif
    
    GPIO_SetPullUp(LCD_CTRL_PORT,LCDE_BIT|LCDRW_BIT|LCDRS_BIT,ENABLE);
    GPIO_SetOutput(LCD_CTRL_PORT,LCDE_BIT|LCDRW_BIT|LCDRS_BIT);

}

/*******************************************************************************
* Function Name  : LCD_Configuration
* Description    : Configures LCD
* Input          : None.
* Return         : None.
*******************************************************************************/
void LCD_Configuration(void)
{
    LCD_IO();

    LCDE = 0;
    LCDRW = 0;
    LCDRS = 0;

#ifdef BUS_4BIT
    Set_Data(INIT_WAIT_4BIT);
    E_Pulse();
    Set_Data(INIT_WAIT_4BIT);
    E_Pulse();
    Set_Data(INIT_WAIT_4BIT);
    E_Pulse();
    Set_Data(INIT_4BIT_BUS);
    E_Pulse();
    Send_To_LCD(FUNCTION_SET | CONFIG_4BIT_BUS | CONFIG_2_LINES | CONFIG_5X7_DOT, LCD_COMMAND);
#else
    E_Pulse();
    Set_Data(INIT_WAIT_8BIT);
    E_Pulse();
    Set_Data(INIT_WAIT_8BIT);
    E_Pulse();
    Set_Data(INIT_WAIT_8BIT);
    E_Pulse();
    Send_To_LCD(FUNCTION_SET | CONFIG_8BIT_BUS | CONFIG_2_LINES | CONFIG_5X7_DOT, LCD_COMMAND);
#endif

    // enable display
    Send_To_LCD(DISP_ON, LCD_COMMAND);

    // clear display
    Send_To_LCD(CLR_DISP, LCD_COMMAND);
    Delay_LCD(3000);
}

void Delay_LCD(uint32_t time)
{
    volatile uint32_t i = 0;

    for (i = 0; i < time * 10U; ++i);
}
